﻿/*
 Function:		Color
				(See Dlg_color.png)

 Parameters: 
				Color	- Initial color and output in RGB format.
				hGui	- Optional handle to parents Gui. Affects dialog position.
  
 Returns:	
				False if user canceled the dialog or if error occurred	
 */ 
Dlg_Color(ByRef Color, hGui=0){ 
  ;covert from rgb
    clr := ((Color & 0xFF) << 16) + (Color & 0xFF00) + ((Color >> 16) & 0xFF)

;~ typedef struct {
  ;~ DWORD        lStructSize;		0		uint	ptr
  ;~ HWND         hwndOwner;		ptr		ptr		ptr
  ;~ HWND         hInstance;		2ptr	ptr		ptr
  ;~ COLORREF     rgbResult;		3ptr	uint	ptr
  ;~ COLORREF     *lpCustColors;	4ptr	ptr		ptr
  ;~ DWORD        Flags;			5ptr	uint	uint
  ;~ LPARAM       lCustData;		6ptr	ptr		ptr
  ;~ LPCCHOOKPROC lpfnHook;			7ptr	ptr		ptr
  ;~ LPCTSTR      lpTemplateName;	8ptr	ptr		ptr
;~ } CHOOSECOLOR, *LPCHOOSECOLOR;	9ptr

    VarSetCapacity(CHOOSECOLOR, 9*A_PtrSize, 0), VarSetCapacity(CUSTOM, 64, 0)
     NumPut(9*A_PtrSize,		CHOOSECOLOR, 0,"Uint")      ; DWORD lStructSize 
     NumPut(hGui,		CHOOSECOLOR, A_PtrSize,"UPtr")      ; HWND hwndOwner (makes dialog "modal"). 
     NumPut(clr,		CHOOSECOLOR, 3*A_PtrSize,"Uint")     ; clr.rgbResult 
     NumPut(&CUSTOM,	CHOOSECOLOR, 4*A_PtrSize,"UPtr")     ; COLORREF *lpCustColors
     NumPut(0x00000103,CHOOSECOLOR, 5*A_PtrSize,"Uint")     ; Flag: CC_ANYCOLOR || CC_RGBINIT || CC_FULLOPEN

    nRC := DllCall("comdlg32\ChooseColorW", "Ptr", &CHOOSECOLOR)  ; Display the dialog. 
    if (errorlevel <> 0) || (nRC = 0)
       return  false 

    clr := NumGet(CHOOSECOLOR, 3*A_PtrSize,"Uint") 
    
    oldFormat := A_FormatInteger 
    SetFormat, integer, hex  ; Show RGB color extracted below in hex format. 

 ;convert to rgb 
    Color := (clr & 0xff00) + ((clr & 0xff0000) >> 16) + ((clr & 0xff) << 16) 
    StringTrimLeft, Color, Color, 2 
    loop, % 6-strlen(Color) 
		Color=0%Color% 
    Color=0x%Color% 
    SetFormat, integer, %oldFormat% 
	return true
}

/*
 Function:  Font
			 (See Dlg_font.png)				

 Parameters:
            Name	- Initial font,  output.
            Style	- Initial style, output.
            Color	- Initial text color, output.
			Effects	- Set to false to disable effects (strikeout, underline, color).
            hGui	- Parent's handle, affects position.

  Returns:
            False if user canceled the dialog or if error occurred.
 */
Dlg_Font(ByRef Name, ByRef Style, ByRef Color, Effects=true, hGui=0) {

   LogPixels := DllCall("GetDeviceCaps", "Ptr", DllCall("GetDC", "Ptr", hGui), "uint", 90)	;LOGPIXELSY
   Effects := 0x043 + (Effects ? 0x100 : 0)  ;CF_EFFECTS = 0x100, CF_SCREENFONTS=1, CF_INITTOLOGFONTSTRUCT = 0x40 CF_INACTIVEFONTS = 0x02000000

;~ typedef struct tagLOGFONT {
  ;~ LONG  lfHeight;					0		uint
  ;~ LONG  lfWidth;						4		uint
  ;~ LONG  lfEscapement;				8		uint
  ;~ LONG  lfOrientation;				12		uint
  ;~ LONG  lfWeight;					16		uint
  ;~ BYTE  lfItalic;					20		uint
  ;~ BYTE  lfUnderline;					21		b
  ;~ BYTE  lfStrikeOut;					22		b
  ;~ BYTE  lfCharSet;					23		b
  ;~ BYTE  lfOutPrecision;				24		b
  ;~ BYTE  lfClipPrecision;				25		b
  ;~ BYTE  lfQuality;					26		b
  ;~ BYTE  lfPitchAndFamily;			27		b
  ;~ TCHAR lfFaceName[LF_FACESIZE];		28		32
;~ } LOGFONT, *PLOGFONT;				92
	
	VarSetCapacity(LOGFONT, 92, 0)
	
   ;set initial name
   DllCall("RtlMoveMemory", "Ptr", &LOGFONT+28, "Ptr", &Name, "Uint", 64)

   ;convert from rgb  
   clr := ((Color & 0xFF) << 16) + (Color & 0xFF00) + ((Color >> 16) & 0xFF)

   ;set intial data
   if InStr(Style, "bold")
      NumPut(700, LOGFONT, 16)

   if InStr(Style, "italic")
      NumPut(255, LOGFONT, 20, 1)

   if InStr(Style, "underline")
      NumPut(1, LOGFONT, 21, 1)
   
   if InStr(Style, "strikeout")
      NumPut(1, LOGFONT, 22, 1)

   if RegExMatch(Style, "s[1-9][0-9]*", s){
      StringTrimLeft, s, s, 1      
      s := -DllCall("MulDiv", "int", s, "int", LogPixels, "int", 72)
      NumPut(s, LOGFONT, 0, "Int")			; set size
   }
   else  NumPut(16, LOGFONT, 0)         ; set default size

;~ typedef struct {
  ;~ DWORD        lStructSize;	0			uint
  ;~ HWND         hwndOwner;	ptr			ptr
  ;~ HDC          hDC;			2ptr		ptr
  ;~ LPLOGFONT    lpLogFont;	3ptr		ptr
  ;~ INT          iPointSize;	4ptr		uint
  ;~ DWORD        Flags;		4 + 4ptr	uint
  ;~ COLORREF     rgbColors;	8 + 4ptr	uint
  ;~ LPARAM       lCustData;	8 +5ptr		ptr
  ;~ LPCFHOOKPROC lpfnHook;		8 +6ptr		ptr
  ;~ LPCTSTR      lpTemplateName;	8+7ptr 	ptr
  ;~ HINSTANCE    hInstance;	8+8ptr		ptr
  ;~ LPTSTR       lpszStyle;	8+9ptr		ptr
  ;~ WORD         nFontType;	8+10ptr		ushort
  ;~ INT          nSizeMin;		8+11ptr		uint
  ;~ INT          nSizeMax;		12+11ptr	uint
;~ } CHOOSEFONT, *LPCHOOSEFONT;	16+11ptr

	size := 16 + 11*A_PtrSize
   VarSetCapacity(CHOOSEFONT, size, 0)
    ,NumPut(size,		 CHOOSEFONT, 0)		; DWORD lStructSize
    ,NumPut(hGui,    CHOOSEFONT, A_PtrSize,"UPtr")		; HWND hwndOwner (makes dialog "modal").
    ,NumPut(&LOGFONT,CHOOSEFONT, 3*A_PtrSize,"UPtr")	; LPLOGFONT lpLogFont
    ,NumPut(Effects, CHOOSEFONT, 4 + 4*A_PtrSize,"Uint")	
    ,NumPut(clr,	 CHOOSEFONT, 8 + 4*A_PtrSize,"Uint")	; rgbColors

   r := DllCall("comdlg32\ChooseFontW", "Ptr", &CHOOSEFONT)  ; Display the dialog.
   if !r
		return false

  ;font name
	VarSetCapacity(Name, 64)
	DllCall("RtlMoveMemory", "wstr", Name, "Ptr", &LOGFONT + 28, "Uint", 64)
	Style := "s" NumGet(CHOOSEFONT, 4*A_PtrSize) // 10

  ;color
	old := A_FormatInteger
	SetFormat, integer, hex                      ; Show RGB color extracted below in hex format.
	Color := NumGet(CHOOSEFONT, 8 + 4*A_PtrSize)
	SetFormat, integer, %old%

  ;styles
	Style =
	VarSetCapacity(s, 3)
	DllCall("RtlMoveMemory", "Ptr", &s, "Ptr", &LOGFONT + 20, "Uint", 3)

	if NumGet(LOGFONT, 16) >= 700
	  Style .= "bold "

	if NumGet(LOGFONT, 20, "UChar")
      Style .= "italic "
   
	if NumGet(LOGFONT, 21, "UChar")
      Style .= "underline "

	if NumGet(LOGFONT, 22, "UChar")
      Style .= "strikeout "

	s := NumGet(LOGFONT, 0, "Int")
	Style .= "s" Abs(DllCall("MulDiv", "int", abs(s), "int", 72, "int", LogPixels))

 ;convert to rgb 
	oldFormat := A_FormatInteger 
    SetFormat, integer, hex  ; Show RGB color extracted below in hex format. 

    Color := (Color & 0xff00) + ((Color & 0xff0000) >> 16) + ((Color & 0xff) << 16) 
    StringTrimLeft, Color, Color, 2 
    loop, % 6-strlen(Color) 
		Color=0%Color% 
    Color=0x%Color% 
    SetFormat, integer, %oldFormat% 

   return 1
}