﻿/* =========================================================================
	MMenu													by Miodrag Milic
													 miodrag.milic@gmail.com	
	Interface
	---------
						   
	MMenu_Create	( _options_ )
	MMenu_Destroy	( menu )

	MMenu_Add		( menu _,title, icon, position, options _)
	MMenu_Set		( menu, item, title _,icon, options_ )
	MMenu_Remove	( menu, item )

	MMenu_Show		( menu, X, Y, OnClick _,OnSelect, OnInit, OnUninit _)
	MMenu_Hide		()
	MMenu_About		()
	
	MMenu_Count		 ( menu )
	MMenu_GetPosition(pMenu, ByRef X, ByRef Y) {


	See documentation, in MMenu.htm for more details

============================================================================
*/

MMenu_InitArrays()
{
	global
	QC_MMenus := Object()
	MMenu_Handles := Object()
	MMenu_MenuByHandle := Object()
	MMenu_MenuProps := Object()
	MMenu_ItemProps := Object()
	MMenu_ItemIDs := Object()
	return
}

MMenu_Create( pOptions="", LightMenu = 0 ) {
	local menu, hMenu
	;Menu info is kept in the MMenu_aMenu array
	;MMenu_aMenu_0_ keeps the number of menus

	MMenu_Menus_Count++
	menu := MMenu_Menus_Count

	;Create the menu and associate its handle.
	hMenu := API_CreatePopUpMenu()

	;~ if (hMenu < 0)		;convert to unsigned integer
		;~ hMenu += 4294967296
	MMenu_Handles[menu] := hMenu
	MMenu_MenuProps[menu,"light"] := LightMenu
	MMenu_MenuByHandle[hMenu] := menu
	
	MMenu_parseMenuOptions( menu, pOptions )

	MMenu_setMaxHeight(menu)
	if !LightMenu
		MMenu_setBackground(menu)
	MMenu_setMenuStyle( menu, 0x80000000 )

	return menu
}

MMenu_Count( pMenu ) {
	global
	return API_GetMenuItemCount( MMenu_Handles[pMenu] )
}

MMenu_GetRealHandle(pMenu)
{
	global
	return MMenu_Handles[pMenu]
}


;------------------------------------------------------------------------

MMenu_onDraw(wparam, lparam){
	local clr, menu, obj, NoText, NoIco, LightMenu, itemColor
	if (wparam != 0)	;means that message is not intendent for menu
		return 0
	
	DRAWITEM_GetA("MMenu_di", lparam)

	NoText := MMenu_ItemProps[MMenu_di_itemID,"notext"]
	NoIco := MMenu_ItemProps[MMenu_di_itemID,"noico"]
	menu := MMenu_ItemProps[MMenu_di_itemID,"parent"]
	LightMenu := MMenu_MenuProps[menu,"light"]
	itemColor := MMenu_ItemProps[MMenu_di_itemID,"color"]

	if !MMenu_ItemProps[MMenu_di_itemID,"grayed"]
	{
		if (LightMenu || !itemColor)
			MMenu_ItemProps[MMenu_di_itemID,"color"] := itemColor := 0x000000

		obj := API_SelectObject( MMenu_di_hDC, GetBrushHandle( itemColor ) )
		DeleteObject(obj)
		API_SetTextColor(MMenu_di_hDC, itemColor)
	}
 	
	if (!LightMenu && !NoIco)
	{
		if NoText
			API_DrawIconEx(MMenu_di_hDC, 2, 2, MMenu_di_itemData, 0, 0, 0, 0, 3) ;MMenu_di_NORMAL=3	  = MMenu_di_MASK | MMenu_di_IMAGE (1 | 2)
		else
			API_DrawIconEx(MMenu_di_hDC, 4, MMenu_di_rcItem_Top+MMenu_Icon_offset, MMenu_di_itemData, 0, 0, 0, 0, 3) ;MMenu_di_NORMAL=3	  = MMenu_di_MASK | MMenu_di_IMAGE (1 | 2)
	}
	
	return 1
	;(API_GetMenuCheckMarkDimensions() & 0xFFFF) + 
}

MMenu_setMenuStyle( pMenu, dwStyle ) {
   local  hMenu := MMenu_Handles[pMenu]

   MMenu_mi_fMask      := 0x80000010      ;MIM_STYLE + MIM_APPLYTOSUBMENUS
   MMenu_mi_dwStyle   := dwStyle

   MENUINFO_Set("MMenu_mi")
   API_SetMenuInfo( hMenu, &MMenu_mi )
}

MMenu_GetPosition(pMenu, ByRef X, ByRef Y, pSelection=false) {
	local hMenu := MMenu_Handles[pMenu]
	local res := 1
	local item

	item = 0
	if (pSelection) 
		loop, % API_GetMenuItemCount(hMenu) 
			if API_GetMenuState( hMenu, A_Index-1, 0x400) & 0x80     ;MF_HILITE= 0x80
			{
				item := A_Index-1
				break
			}

	RECT_Set("MMenu_rect")
	res := DllCall("GetMenuItemRect", "Ptr", 0, "Ptr", hMenu, "uint", item, "Ptr", &MMenu_rect_c)
	RECT_Get("MMenu_rect")
	
	X := MMenu_rect_left
	Y := MMenu_rect_top

	return %res%
}
 
MMenu_GetItemIDbyPos(pMenu,pPos)
{
	local hId,item_id
	hId := MMenu_findItemByPos( pMenu, pPos )
	item_id := MMenu_ItemProps[hId,"ID"]
	return item_id
}
 
;-----------------------------------------------------------------------------
; Add new menu item with specified attributes above pItem
; If pItem doesn't exist, or if it equals to 0, the new item will be appended 
;
; Returns false if pMenu is invalid or true if item is added.
MMenu_Add( pMenu, pTitle="", pIcon="", pItem=0, pOptions="",NoText = 0,NoIco = 0 ) {
	local hMenu := MMenu_Handles[pMenu]
	local idx, res, sID

	if (hMenu = "")
		return 0				;ERR_MENU
	
	MMenu_ItemsCount++
	sID := MMenu_ItemsCount
	
	if (pIcon . pTitle . pOptions . pItem = 0)	;check for separator
		pOptions := "s"

	;set the item data
	MMenu_ItemProps[sID,"parent"] := pMenu
	MMenu_ItemProps[sID,"title"] := pTitle
	MMenu_ItemProps[sID,"notext"] := NoText
	MMenu_ItemProps[sID,"noico"] := NoIco


	;get item
	idx := MMenu_getItemIdx( pMenu, pItem )
	if NoText
		res := API_InsertMenu(hMenu, idx, 0x0, sID)  ;MF_BYCOMMAND = 0x0
	else
		res := API_InsertMenu(hMenu, idx, 0x0, sID, pTitle)  ;MF_BYCOMMAND = 0x0

	;set icon
	if !NoIco
		MMenu_setItemIcon(pMenu, sID, pIcon)

	;set item options
	MMenu_setItemOptions( pMenu, sID, pOptions )
	
	return res
}

;------------------------------------------------------------------------

MMenu_Set( pMenu, pItem, pTitle="", pIcon="", pOptions="" ){
	local hMenu := MMenu_Handles[pMenu]
	local idx := MMenu_getItemIdx( pMenu, pItem )
	local r	:= 1
	
	
	if !idx				;if invalid item
		return 0

	if (hMenu = "")
		return 0

	if (pTitle != "")
	{
		if (pTitle = " ")
			pTitle =

		r := MMenu_setItemTitle(pMenu, idx, pTitle)
	}

	if (pIcon != "") {
		if (pIcon = " ")
			pIcon = 
		r := MMenu_setItemIcon(pMenu, idx, pIcon) AND r

	}

	if (pOptions != "")
		r := MMenu_setItemOptions( pMenu, idx, pOptions ) AND r
	
	return r ? 1 : 0
}

;- - - - - - - - - - - - - - - -  

MMenu_setItemTitle(pMenu, pIdx, pTitle) {
	local hMenu := MMenu_Handles[pMenu]

	MMenu_ItemProps[pIdx,"title"] := pTitle
	
	MMenu_mii_fMask := 0x40				;MIIM_STRING
	MMenu_mii_dwTypeData := &pTitle
	MMenu_mii_cch := StrLen(pTitle)
 
	MENUITEMINFO_Set("MMenu_mii")
 	return, API_SetMenuItemInfo(hMenu, pIdx, false, &MMenu_mii )
}

;- - - - - - - - - - - - - - - -  
; Should set for picon="" too
;
MMenu_setItemIcon(pMenu, pIdx, pIcon) {
	local hMenu		:= MMenu_Handles[pMenu]
	local iconSize	:= MMenu_MenuProps[pMenu,"iconSize"]
	local sub		:= MMenu_ItemProps[pIdx,"submenu"]
	local hSub		:= MMenu_Handles[sub]
	local res := 1
	local hicon

	;remove old icon
	;if (MMenu_ItemProps[pIdx,"separator"])
	
	if MMenu_ItemProps[pIdx,"hIcon"]
		MMenu_destroyIcon( MMenu_ItemProps[pIdx,"hIcon"] )

	if pIcon
	{
		MMenu_ItemProps[pIdx,"icon"] := pIcon
		MMenu_ItemProps[pIdx,"hIcon"] := MMenu_mii_dwItemData := MMenu_loadIcon(pIcon, iconSize) 
	}
	else
		MMenu_ItemProps[pIdx,"hIcon"] := MMenu_mii_dwItemData := ""
	
	if (MMenu_mii_dwItemData != 0)
		 MMenu_MenuProps[pMenu,"hasIcons"] := true
	else res := 0

	MMenu_mii_fMask		:= 0x80 | 0x20	;MIIM_BITMAP | MIIM_DATA
	MMenu_mii_hbmpItem	:= -1
	MENUITEMINFO_Set("MMenu_mii")
	res := API_SetMenuItemInfo(hMenu, pIdx, false, &MMenu_mii) & res
	
	return res
}

;- - - - - - - - - - - - - - - -  

MMenu_setItemOptions(pMenu, pIdx, pOptions) {
	local hMenu := MMenu_Handles[pMenu], sub

	;parse options
	MMenu_parseItemOptions(pIdx, pOptions, pMenu)

	;get previous item state
	MMenu_mii_fMask := 0x1	;MMenu_miiM_STATE
	MENUITEMINFO_Set("MMenu_mii")
	API_GetMenuItemInfo(hMenu, pIdx, false, &MMenu_mii )
	MENUITEMINFO_Get("MMenu_mii")


;TYPE OPTIONS, only one at the time can be set
	if (MMenu_ItemProps[pIdx,"separator"])
		 MMenu_mii_fType :=  0x800				;MFT_SEPARATOR

	else if (MMenu_ItemProps[pIdx,"break"])
			MMenu_mii_fType := 	MMenu_ItemProps[pIdx,"break"] = 2 ? 0x20 : 0x40		;MFT_MENUBARBREAK :	MFT_MENUBREAK

		 else MMenu_mii_fType := 0	


;STATE OPTIONS, more can be set
	if MMenu_ItemProps[pIdx,"grayed"]
		 MMenu_mii_fState |= 0x3		;MFS_GRAYED
	else MMenu_mii_fState &= ~0x3

	if (MMenu_ItemProps[pIdx,"check"])
		 MMenu_mii_fState |= 0x8		;MFS_CHECKED
	else MMenu_mii_fState &= ~0x8

	if (MMenu_ItemProps[pIdx,"default"])
		 MMenu_mii_fState |= 0x1000	    ;MFS_DEFAULT
	else MMenu_mii_fState &= ~0x1000	


;SUBMENU OPTION
	sub  := MMenu_ItemProps[pIdx,"submenu"]
	if (sub != "") {
		 MMenu_mii_hSubmenu := MMenu_Handles[sub]
		 MMenu_MenuProps[sub,"parent"] := pMenu			;!!! if the user adds the same submenu into menu multiple times... who cares
	}
 	else MMenu_mii_hSubmenu := 0


	;set type options
	MMenu_mii_fMask := 0x100			;MIIM_FTYPE	
	MENUITEMINFO_Set("MMenu_mii")
	API_SetMenuItemInfo(hMenu, pIdx, false, &MMenu_mii )

	;set state options
	MMenu_mii_fMask := 0x1	;MIIM_STATE
	MENUITEMINFO_Set("MMenu_mii")
 	API_SetMenuItemInfo(hMenu, pIdx, false, &MMenu_mii )

	;set submenu options
	MMenu_mii_fMask := 0x4				;MIIM_SUBMENU = 0x4
	MENUITEMINFO_Set("MMenu_mii")
 	API_SetMenuItemInfo(hMenu, pIdx, false, &MMenu_mii )

	return 1
}


;------------------------------------------------------------------------
; return false on failure, true on succes
MMenu_Remove( pMenu, pItem=0 ) { 
	local hMenu := MMenu_Handles[pMenu]
	local idx := MMenu_getItemIdx( pMenu, pItem )
	local res

 	res := API_RemoveMenu(hMenu, idx, 0x0)		;MF_BYCOMMAND=0x0	
	MMenu_freeItem( idx )

	return res
}

;------------------------------------------------------------------------
; returns items internal id, or 0 on failure
;
MMenu_getItemIdx( pMenu, pItem ) {


	if InStr(pItem, A_Space, false, 0)	{
		StringTrimRight, pItem, pItem, 1
		return MMenu_findItemByTitle( pMenu, pItem)
	}

	if pItem is integer
		 return MMenu_findItemByPos( pMenu, pItem )
    else return MMenu_findItemByID( pMenu, pItem )
}

;------------------------------------------------------------------------
; return 0 on failure
MMenu_findItemByID( pMenu, pID ) {
	local res

	res := MMenu_ItemIDs[pID]
	if res =
		return 0

	return res
}

;- - - - - - - - - - - - - - - -  
; return 0 on failure
MMenu_findItemByTitle( pMenu, pTitle ) {
	local hMenu := MMenu_Handles[pMenu]
	local cnt := API_GetMenuItemCount(hMenu)
	local buf

	if pTitle = 
		return 0

	VarSetCapacity(buf, 512)
	loop, %cnt%
	{
	  	 DllCall("GetMenuString", "Ptr", hMenu, "uint", A_Index-1, "str", buf, "uint", 512, "uint", 0x400) ;MF_BYPOSITION = 0x400
		 if (buf = pTitle)
			return MMenu_GetMenuItemID(hMenu, A_Index-1)
	}

	return 0
}

;- - - - - - - - - - - - - - - -  
; return 0 on failure
MMenu_findItemByPos( pMenu, pPos ) {
	local hMenu := MMenu_Handles[pMenu]
	local cnt := API_GetMenuItemCount(hMenu)
	local res

	if pPos <= 0
		return 0

	if pPos > cnt
		return 0

	res := MMenu_GetMenuItemID(hMenu, --pPos)
	if res = -1
		return 0
	
	return res

}

;this works for submenus too. Original OS function returns -1 for submenus.
MMenu_getMenuItemID(hMenu, pos){
	global

	MMenu_mii_fMask := 0x2				;MIIM_ID	
	MMenu_mii_wID := 0
	MENUITEMINFO_Set("MMenu_mii")
	API_GetMenuItemInfo(hMenu, pos, true, &MMenu_mii )
	MENUITEMINFO_Get("MMenu_mii")

	return MMenu_mii_wID
}

;------------------------------------------------------------------------

MMenu_Destroy( pMenu ) {
	local hMenu := MMenu_Handles[pMenu]
	ret := API_DestroyMenu( hMenu )
	MMenu_freeMenu( pMenu)
}

;------------------------------------------------------------------------

MMenu_freeMenu( pMenu ){
	local hMenu := MMenu_Handles[pMenu]
	local sub

	loop,%MMenu_ItemsCount%
		if (MMenu_ItemProps[A_Index,"parent"] = pMenu)
		{
			sub := MMenu_ItemProps[A_Index,"submenu"]
			if ( sub != "")
				MMenu_freeMenu( sub )
			
			MMenu_freeItem( A_Index )
		}
	MMenu_Handles.Remove(pMenu,"")
	MMenu_MenuProps.Remove(pMenu,"")
	MMenu_MenuByHandle.Remove(hMenu,"")
}

;- - - - - - - - - - - - - - - -  

MMenu_freeItem(pIdx){
	local id :=	MMenu_ItemProps[pIdx,"ID"]
	local ico_h := MMenu_ItemProps[pIdx,"hIcon"]
	
	if ico_h
		res := MMenu_destroyIcon( ico_h )
	
	MMenu_ItemProps.Remove(pIdx,"")
	MMenu_ItemIDs.Remove(id)
}

;------------------------------------------------------------------------
MMenu_Hide(pMenu=""){
	if pMenu
		menu_handle := MMenu_GetRealHandle(pMenu)
	DllCall("EndMenu", "Ptr", menu_handle)
	return
}


MMenu_parseHandlers( pOptions  ){
	local c, token	

	MMenu_userInit	:= 	MMenu_userUninit := MMenu_userSelect := ""
	MMenu_userMiddle := MMenu_userRight := MMenu_userMenuChar := ""

	Loop, Parse, pOptions, %A_Space%
	{
		StringLeft c, A_LoopField, 1
		StringTrimLeft token, A_LoopField, 1

		if (c="S") {
			MMenu_userSelect := token
			continue
		}

		if (c="I") {
			MMenu_userInit	:= token
			continue
		}
		if (c="U") {
			MMenu_userUninit	:= token
			continue
		}
		if (c="M") {
			MMenu_userMiddle := token
			continue
		}
		if (c="R") {
			MMenu_userRight	:= token
		}

		if (c="C") {
			MMenu_userMenuChar := token
		}
	}

}


MMenu_Show( pMenu, pX, pY, pOnClick, pHandlers="",Trans=255) 
{ 
	local hMenu := MMenu_Handles[pMenu]
	local itemID, flags
	MMenu_trans := Trans
	MMenu_parseHandlers(pHandlers)

	Gui 99:+LastFound +ToolWindow +HwndMMenu_hParent
	
	Gui, 99:Show	; x0 y0 w0 h0
	MMenu_Hide()
	MMenu_MsgMonitor(true)
	itemID := API_TrackPopupMenu(hMenu, 0x0100, pX, pY, MMenu_hParent) ;TPM_RETURNCMD = 0x100
	SetTimer,WatchForMenu,OFF
	MMenu_trans = 
	MMenu_MsgMonitor(false)
	Gui 99:Hide
;	A_LastError=1401 -invalid menu handle

	;if menu is canceled, return
	if itemID = 0
		return


	M_Title		:= MMenu_ItemProps[itemID,"title"]
	M_ID		:= MMenu_ItemProps[itemID,"ID"]
	M_Menu		:= MMenu_ItemProps[itemID,"parent"]

	if pOnClick
		GoSub %pOnClick%
	return M_ID
}


;------------------------------------------------------------------------

MMenu_parseMenuOptions( pMenu, pOptions ){
	local c, token, LightMenu

	;defaults
	MMenu_MenuProps[pMenu,"iconSize"] := 32
	MMenu_MenuProps[pMenu,"textOffset"] := 5
	MMenu_MenuProps[pMenu,"color"] := 0xFFFFFF
	MMenu_MenuProps[pMenu,"textcolor"] := 0
	MMenu_MenuProps[pMenu,"h_offset"] := 0
	LightMenu := MMenu_MenuProps[pMenu,"light"]

	Loop, Parse, pOptions, %A_Space%
	{
		StringLeft c, A_LoopField, 1
		StringTrimLeft token, A_LoopField, 1

		;icon size
		if (c="S")	{
			if LightMenu
				MMenu_MenuProps[pMenu,"iconSize"] := 0
			else
				MMenu_MenuProps[pMenu,"iconSize"] := token
			continue
		}

		if (c="O")	{
			MMenu_MenuProps[pMenu,"textOffset"] := token
			continue
		}

		if (c="H")	{
			if token < 100
				token := 100 

			MMenu_MenuProps[pMenu,"maxHeight"] := token
			continue
		}

		if (c="C")	{
			MMenu_MenuProps[pMenu,"color"] := "0x" token
			continue
		}

		if (c="T")	{
			MMenu_MenuProps[pMenu,"textcolor"]  := "0x" token
			continue
		}
		
		if (c="I")	{
			MMenu_MenuProps[pMenu,"h_offset"] := token
			continue
		}
	}
}

;------------------------------------------------------------------------

MMenu_setMaxHeight( pMenu ) {
	local  hMenu := MMenu_Handles[pMenu]

	MMenu_mi_fMask	:= 0x1		;MIM_MAXHEIGHT
	MMenu_mi_cyMax	:= MMenu_MenuProps[pMenu,"maxHeight"]

	MENUINFO_Set("MMenu_mi")
	API_SetMenuInfo( hMenu, &MMenu_mi )
}

;- - - - - - - - - - - - - - - -  

MMenu_parseItemOptions( idx, pOptions, pMenu  )
{
	local c, token, bRemove	
	
	;default values
	MMenu_ItemProps[idx,"color"] := MMenu_MenuProps[pMenu,"textcolor"]
    Loop, Parse, pOptions, %A_Space% ;, + ;Were omitted
	{
		StringLeft c, A_LoopField, 1
		StringTrimLeft token, A_LoopField, 1

		if (c="-") {
			bRemove := true
			c := chr(*&token) 
		}

		if (c="I")	{
			
			MMenu_ItemProps[idx,"ID"] := token
			MMenu_ItemIDs[token] := idx
			continue
		}

		if (c="S")	{
			MMenu_ItemProps[idx,"separator"] := bRemove ?  "" : true
			continue
		}

		if (c="D")	{
			MMenu_ItemProps[idx,"default"] := bRemove ?  "" : true
			continue
		}

		if (c="B")	{
			MMenu_ItemProps[idx,"break"]	:= bRemove ?  "" : (token = "" ? 1 : 2)
			continue
		}

		if (c="G")	{
			MMenu_ItemProps[idx,"grayed"] := bRemove ?  false : true
			continue
		}

		if (c="M")	{
			MMenu_ItemProps[idx,"submenu"]	:= bRemove ?  "" : token
			continue
		}

		if (c="C")	{
			MMenu_ItemProps[idx,"check"] := bRemove ?  "" : true
			continue
		}

		if (c="T")	{
			MMenu_ItemProps[idx,"color"] := bRemove ?  MMenu_MenuProps[pMenu,"textcolor"] : "0x" token
			continue
		}
	}
}

;------------------------------------------------------------------------
MMenu_msgMonitor( on ){
	local WM_MENUSELECT		= 0x11F
	local WM_MEASUREITEM	= 0x2C
	local WM_DRAWITEM		= 0x2B
	local WM_ENTERMENULOOP	= 0x211
	local WM_INITMENUPOPUP	= 0x117
	local WM_UNINITMENUPOPUP= 0x125
	local WM_MENUSELECT		= 0x11F
	local WM_EXITMENULOOP	= 0x212
	local WM_MENUCOMMAND	= 0x126
	local WM_CONTEXTMENU	= 0x7b
	local WM_MBUTTONDOWN	= 0x207
	local WM_MENUCHAR		= 0x120



	static oldMeasure, oldDraw, oldrbutton, oldMButton, oldMenuChar

	if (on)	{
		OnMessage(WM_ENTERMENULOOP, "MMenu_OnEnterLoop")
		OnMessage(WM_MEASUREITEM,	"MMenu_OnMeasure")
		OnMessage(WM_DRAWITEM,		"MMenu_OnDraw")
		OnMessage(WM_MENUSELECT,    "MMenu_OnSelect")
		OnMessage(WM_INITMENUPOPUP, "MMenu_OnInit")
		OnMessage(WM_UNINITMENUPOPUP,"MMenu_OnUninit")
		OnMessage(WM_CONTEXTMENU,	"MMenu_OnRButtonDown")
		OnMessage(WM_MBUTTONDOWN,	"MMenu_OnMButtonDown")
		OnMessage(WM_MENUCHAR,		"MMenu_OnMenuChar")
	}
	else {
		OnMessage(WM_ENTERMENULOOP, "")
		OnMessage(WM_MEASUREITEM,	"")
		OnMessage(WM_DRAWITEM,		"")
		OnMessage(WM_INITMENUPOPUP, "")
		OnMessage(WM_MENUSELECT, 	"")
		OnMessage(WM_EXITMENULOOP, 	"")
		OnMessage(WM_UNINITMENUPOPUP, "")
		OnMessage(WM_CONTEXTMENU, 	"")
		OnMessage(WM_MBUTTONDOWN, 	"")
		OnMessage(WM_MENUCHAR,		"")
	}
}

;--------------------------------------------------------------------------------

MMenu_onEnterLoop(){
	return 1
}

;- - - - - - - - - - - - - - - -  
;MIM_BACKGROUND = 2
MMenu_setBackground(pmenu){
	local hMenu := MMenu_Handles[pMenu]
	local pColor := MMenu_MenuProps[pMenu,"color"]
	
	MMenu_mi_fMask	 := 0x2		;MIM_BACKGROUND
	if !pColor
		MMenu_MenuProps[pMenu,"color"] := pColor := "0x" . MENU_COLOR_DEFAULT
	MMenu_mi_hbrBack := GetBrushHandle( pColor )
	MENUINFO_Set("MMenu_mi")
	API_SetMenuInfo( hMenu, &MMenu_mi )
}



;- - - - - - - - - - - - - - - -  

MMenu_onMeasure(wparam, lparam) {
	local iconSize, textOffset, menu, NoText, NoIco, w, h, iconOffset, ismenu
	local idx := ExtractIntegerAtAddr(lparam, 8,  1)	;pointer to the MEASUREITEMSTRUCT is in lparam

	menu  := MMenu_ItemProps[idx,"parent"]
	NoText := MMenu_ItemProps[idx,"notext"]
	NoIco := MMenu_ItemProps[idx,"noico"]
	iconSize := MMenu_MenuProps[menu,"iconSize"]
	textOffset := MMenu_MenuProps[menu,"textOffset"]
	iconOffset := MMenu_MenuProps[menu,"h_offset"]
	ismenu := MMenu_ItemProps[idx,"submenu"]

	;~ if (MMenu_ItemProps[idx,"hIcon"] = 0 || MMenu_ItemProps[idx,"hIcon"] = "")
		;~ if !MMenu_MenuProps[menu,"hasIcons"]         	;if item  has no icon at all menu has no icons at all, just return
				;~ return 0
		;~ else{
			;~ InsertIntegerAtAddr(iconsize + textOffset, lParam, 12)		;else put the width and offset only (without this, windows displays non icon titles bugy)
			;~ return 1
		;~ }

	MMenu_Icon_offset := iconOffset
	if NoText
	{
		if ismenu
			w := iconSize - 10
		else
			w := iconSize -21
		h := iconSize + 2
	}
	else if NoIco
	{
		w := 0
		h := 0
	}
	else
	{
		w := iconSize + textOffset
		h := iconSize + iconOffset*2
	}
	InsertIntegerAtAddr(w, lParam, 12)
	InsertIntegerAtAddr(h, lParam, 16)
	
	
	return 1
}

;- - - - - - - - - - - - - - - -  

MMenu_onMenuChar(wparam, lparam){
	global
	M_CMENU := M_CHAR :=

	if MMenu_userMenuChar =
		return 

	M_CMENU := MMenu_MenuByHandle[lparam]
	M_CHAR  := wparam & 0xFFFF

	GoSub %MMenu_userMenuChar%
	return 3<<16
}

;- - - - - - - - - - - - - - - -  

MMenu_onInit(wparam, lparam){
	global
	M_MENU := ""
	
	if MMenu_userInit =
		return 

	if wparam < 0		;convert to unsigned integer
		wparam += 4294967296

	- try to fix the deactivation bug
	M_MENU := MMenu_MenuByHandle[wparam]

	GoSub %MMenu_userInit%
}

;- - - - - - - - - - - - - - - -  
;wParam
;
;The low-order word specifies the menu item or submenu index. 
;If the selected item is a command item, this parameter contains the identifier of the menu item. 
;If the selected item opens a drop-down menu or submenu, this parameter contains the index of the 
;drop-down menu or submenu in the main menu, and the lParam parameter contains the handle to the 
;main (clicked) menu; use the GetSubMenu function to get the menu handle to the drop-down menu or submenu.
;
MMenu_onSelect(wparam, lparam){
	local idx  := wparam & 0xFFFF
	local menuFlag := wparam >> 16 
	local hSub, gg, menu, sub
	
	M_SMENU	:= M_SID := M_STITLE := ""
	
	;lparam = 0 represents dummy message that is sent when user press ESC
	if (lparam = 0) || (MMenu_userSelect = "" && MMenu_userRight = "" && MMenu_userMiddle)
		return 

	if (lparam < 0)		;convert to unsigned integer
		lparam += 4294967296

	if menuFlag in 32912,144			;MF_POPUP and some number for submenus
	{
		hSub := API_GetSubmenu( lparam, idx )
		if hSub < 0		;convert to unsigned integer
			hSub += 4294967296


		menu :=	MMenu_MenuByHandle[lparam]
		sub :=	MMenu_MenuByHandle[hSub]
		loop, %MMenu_ItemsCount%
		{
			if (MMenu_ItemProps[A_Index,"parent"] = menu)
				if (MMenu_ItemProps[A_Index,"submenu"] = sub)
				{
					idx := A_Index
					break
				}
		}
	}		
	
	M_SMENU	:= MMenu_MenuByHandle[lparam]
	M_SID	:= MMenu_ItemProps[idx,"ID"]
	M_STITLE := MMenu_ItemProps[idx,"title"]

	if (MMenu_userSelect != "")
		GoSub %MMenu_userSelect%
	return
}

;- - - - - - - - - - - - - - - -  

MMenu_onUninit(wparam){
	global
	M_MENU := ""
	
	if MMenu_userUninit =
		return 

	if wparam < 0		;convert to unsigned integer
		wparam += 4294967296

	M_MENU := MMenu_MenuByHandle[wparam]
	GoSub %MMenu_userUninit%
}

;- - - - - - - - - - - - - - - -  

MMenu_onRButtonDown(wparam, lparam){
	global
	
	M_MENU := M_ID := M_TITLE := ""
	
	if MMenu_userRight =
		return
		
	M_Title		:= M_STitle
	M_ID		:= M_SID
	M_Menu		:= M_SMenu
	GoSub %MMenu_userRight%
}

;- - - - - - - - - - - - - - - -  

MMenu_OnMButtonDown(wparam, lparam){
	global
	
	M_MENU := M_ID := M_TITLE := ""
	
	if MMenu_userMiddle =
		return 

	M_Title		:= M_STitle
	M_ID		:= M_SID
	M_Menu		:= M_SMenu

	GoSub %MMenu_userMiddle%
}

;--------------------------------------------------------------------------------

#include API_Menu.ahk	

;====================================================================================
; API_Draw.ahk
;=====================================================================================

MMenu_loadIcon(pPath, pSize=0)
{
	idx := InStr(pPath, ":", 0, 0)
	if idx >=4
	{
		resPath := SubStr( pPath, 1, idx-1)
		resIdx  := Substr( pPath, idx+1, 8)
		return IconExtract(resPath,resIdx,pSize)
	}
	if substr(pPath,-2) = "_DF"
	{
		StringReplace,pPath,pPath,_DF
		Return pPath
	}
	if pPath is not Integer					;if .ICO
		return IconLoad(pPath,pSize)
	else if pPath is Integer					;if icon handle were passed
		return IconCopy(pPath,pSize)			
}

;--------------------------------------------------------------------------------

MMenu_destroyIcon(hIcon,f=0) {
	global icons_ext_handles, DF_FOLDER, DF_OPEN_FOLDER
	if f
		ret_val := DllCall("DestroyIcon", "Ptr", hIcon)
	else if (!instr(icons_ext_handles,hIcon) && hIcon != DF_FOLDER && hIcon != DF_OPEN_FOLDER)	;!RegExMatch(icons_ext_handles,"m)\|\s*" . hIcon . "\s*$")
		ret_val := DllCall("DestroyIcon", "Ptr", hIcon)
	return ret_val
}

;--------------------------------------------------------------------------------

API_DrawIconEx( hDC, xLeft, yTop, hIcon, cxWidth, cyWidth, istepIfAniCur, hbrFlickerFreeDraw, diFlags)
{
    return DllCall("DrawIconEx"
            ,"Ptr", hDC
            ,"uint", xLeft
            ,"uint", yTop
            ,"Ptr", hIcon
            ,"int",  cxWidth
            ,"int",  cyWidth
            ,"uint", istepIfAniCur
            ,"Ptr", hbrFlickerFreeDraw
            ,"uint", diFlags )
}

;--------------------------------------------------------------------------------
/*
typedef struct tagDRAWITEMSTRUCT {
  UINT      CtlType;		0
  UINT      CtlID;			4
  UINT      itemID;			8
  UINT      itemAction;		12
  UINT      itemState;		16
  HWND      hwndItem;		20
  HDC       hDC;			20+ptr
  RECT      rcItem;			
  left						20+2ptr
  top						24+2ptr
  right						28+2ptr
  bottom					32+2ptr
  ULONG_PTR itemData;		36+2ptr
} DRAWITEMSTRUCT;
*/
DRAWITEM_GetA(s, adr){
    global

    %s%_itemID      := ExtractIntegerAtAddr(adr,8,  0)
	%s%_itemAction  := ExtractIntegerAtAddr(adr,12, 0)
    %s%_itemState   := ExtractIntegerAtAddr(adr,16, 0)
    %s%_hwndItem    := ExtractIntegerAtAddr(adr,16+A_PtrSize, 0,A_PtrSize)
    %s%_hDC         := ExtractIntegerAtAddr(adr,16+2*A_PtrSize, 0,A_PtrSize)

    %s%_rcItem_Left   := ExtractIntegerAtAddr(adr,16+3*A_PtrSize, 0)
    %s%_rcItem_Top    := ExtractIntegerAtAddr(adr,20+3*A_PtrSize, 0)
    %s%_rcItem_Right  := ExtractIntegerAtAddr(adr,24+3*A_PtrSize, 0)
    %s%_rcItem_Bottom := ExtractIntegerAtAddr(adr,28+3*A_PtrSize, 0)
	%s%_itemData	  := ExtractIntegerAtAddr(adr,32+3*A_PtrSize, 0,A_PtrSize)
}


;-------------------------------------------------------------------------------------------------
API_SetTextColor(hDC, crColor){
	return, DllCall("SetTextColor", "Ptr", hDC, "uint", crColor)
}


API_CreateSolidBrush(crColor){
	return DllCall("CreateSolidBrush", "uint", crColor,"UPtr")
}

API_SelectObject( hDC, hgdiobj ){
    return DllCall("SelectObject", "Ptr", hDC, "Ptr", hgdiobj)
}


RECT_Set(var)
{
	global

	VarSetCapacity(%var%_c, 16 , 0) 
	InsertInteger(%var%_left,   %var%_c, 0)	
	InsertInteger(%var%_top,    %var%_c, 4)	
	InsertInteger(%var%_right,  %var%_c, 8)	  
	InsertInteger(%var%_bottom, %var%_c, 12)	
}

RECT_Get(var)
{
	global

	%var%_left   := ExtractInteger(%var%_c, 0)	
	%var%_top	 := ExtractInteger(%var%_c, 4)	
	%var%_right	 := ExtractInteger(%var%_c, 8)	  
	%var%_bottom := ExtractInteger(%var%_c, 12)	
	%var%_width  := %var%_right - %var%_left
	%var%_height := %var%_bottom - %var%_top
}