﻿PathIsFileSpec( pPath )
{
	return DllCall("Shlwapi\PathIsFileSpecW","Ptr",&pPath)
}

PathSplit( path, ByRef out_args,ByRef out_dir )
{
	out_args := PathGetArgs( path )
	if out_args
		PathRemoveArgs( path )
	out_dir := PathGetDir( path )
	path := PathResolve( path )
	return path
}

PathQuoteSpaces( path )
{
	path := Trim( path )
	if instr( path, A_Space )
		path = "%path%"
	return path
}

PathUnquoteSpaces( path )
{
	path := Trim( path )
	regex = ^"\K.*(?="$)
	while RegExMatch( path, regex, cmd_new )
		path := cmd_new
	return path
}

;removes unallowed characters from file name
PathCleanup( path )
{
	global PATH_SIZE
	VarSetCapacity(qPath,PATH_SIZE,0)
	StrPut( path, &qPath )
	if DllCall("shell32\PathCleanupSpec","Ptr",0,"Ptr",&qPath )
		path := StrGet( &qPath )
	free( qPath )
	return path
}

PathIsNetworkPath( path )
{
	return DllCall( "Shlwapi\PathIsNetworkPathW", "Ptr", &path )
}

PathIsURL( path )
{
	return DllCall( "Shlwapi\PathIsURLW", "Ptr", &path )
}

PathIsDir( path )
{
	return DllCall( "Shlwapi\PathIsDirectoryW", "WStr", path )
}

PathGetDir( path )
{
	path := PathUnquoteSpaces( path )
	if PathIsDir( path )
	{
		DllCall( "Shlwapi\PathRemoveBackslashW", "WStr", path )
		return path
	}
	else
	{
		DllCall( "Shlwapi\PathRemoveFileSpecW", "WStr", path )
		return path
	}
}

PathResolve( path )
{
	path := PathUnquoteSpaces( path )
	if PathIsRelative( path )
		if IsProperPath( path )
			path := PathAppend( A_WorkingDir, path )
	return PathQuoteSpaces( path )
}

IsProperPath( path )
{
	if RegExMatch( path, "^\s*(\.|\.\.)\s*$" )	;paths like "." or ".." considered proper
		return 1
	return !( PathIsFileSpec( path ) || PathIsURL( path ) || InStr( path, "%" ) )
}

PathIsRelative( path )
{
	return DllCall("Shlwapi\PathIsRelativeW","Ptr",&path )
}

PathAppend( path1, path2 )
{
	global PATH_SIZE
	VarSetCapacity(qPath,PATH_SIZE,0)
	StrPut( path1, &qPath )
	if DllCall("Shlwapi\PathAppendW","Ptr",&qPath,"Ptr",&path2 )
		path1 := StrGet( &qPath )
	free( qPath )
	return path1
}

ParseEnvVars(path)
{
	fPos := 1
	loop
		if fPos := RegExMatch( path,"%\K.+?(?=%)", env_var_name, fPos )
		{
			EnvGet, env_var_value, %env_var_name%
			if env_var_value {
				StringReplace,path,path,% "%" env_var_name "%",% env_var_value
				fPos += StrLen( env_var_value ) - 1		;-1 because we removing two %
			}
			else
				fPos += StrLen( env_var_name ) + 1		;+1 because we have one % beyond var name
		}
		else
			break
	return path
}

PathGetArgs(path)
{
	args := DllCall("Shlwapi\PathGetArgsW","str",path,"str")	;return string though function returns pointer
	return args
}

PathRelativeTo(pTo,pFrom = "")
{
	pTo := PathUnquoteSpaces( pTo )
	if (pFrom = "")
		pFrom := A_WorkingDir
	VarSetCapacity(path_out,260*2)
	if IsProperPath( pTo )
		if DllCall("Shlwapi\PathRelativePathToW","Ptr", &path_out,"Ptr",&pFrom,"Int",0x10,"Ptr",&pTo,"int",0)	;FILE_ATTRIBUTE_DIRECTORY = 0x10
			pTo := StrGet( &path_out )
	free( path_out )
	pTo := PathQuoteSpaces( pTo )
	return pTo
}

PathRemoveArgs(ByRef path)
{
	DllCall("Shlwapi\PathRemoveArgsW","str",path)
	Return Path
}

URLEscape( url )
{
	StringReplace,url,url,&euro;,€,1
	StringReplace,url,url,&nbsp;,%A_Space%,1
	StringReplace,url,url,&quot;,",1
	StringReplace,url,url,&amp;,&,1
	StringReplace,url,url,&lt;,<,1
	StringReplace,url,url,&gt;,>,1
	return url
}

MailEscape(ByRef email)
{
	StringReplace, email, email, %A_Space%,`%20,1 
	StringReplace, email, email, %A_Tab%,`%09,1 
	StringReplace, email, email, `n,`%0A,1
	StringReplace, email, email, `r,`%0D,1
	StringReplace, email, email, &,`%26,1
	StringReplace, email, email, ",`%22,1
	StringReplace, email, email, ?,`%3F,1
	StringReplace, email, email, {,`%7B,1
	StringReplace, email, email, },`%7D,1
	return email
}

MailDescape(byref email)
{
	StringReplace, email, email,`%20, %A_Space%,1 
	StringReplace, email, email,`%09,%A_Tab%,1 
	StringReplace, email, email,`%0A,`n,1
	StringReplace, email, email,`%0D,`r,1
	StringReplace, email, email,`%26,&,1
	StringReplace, email, email,`%22, ",1
	StringReplace, email, email,`%3F, ?,1
	StringReplace, email, email,`%7B, {,1
	StringReplace, email, email,`%7D, },1
	return	email 
}